@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/domain.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1+   2009/12/20
+ * Version: 1.7.1+   2010/02/14
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -41,12 +41,18 @@ LIST_HEAD(ccs_domain_list);
 static int ccs_audit_execute_handler_log(struct ccs_execve_entry *ee,
 					 const bool is_default)
 {
+	int error;
 	struct ccs_request_info *r = &ee->r;
+	struct ccs_domain_info *domain = r->domain;
 	const char *handler = ee->handler->name;
 	r->mode = ccs_get_mode(r->profile, CCS_MAC_FILE_EXECUTE);
-	return ccs_write_audit_log(true, r, "%s %s\n",
-				   is_default ? CCS_KEYWORD_EXECUTE_HANDLER :
-				   CCS_KEYWORD_DENIED_EXECUTE_HANDLER, handler);
+	r->domain = ee->previous_domain;
+	error = ccs_write_audit_log(true, r, "%s %s\n",
+				    is_default ? CCS_KEYWORD_EXECUTE_HANDLER :
+				    CCS_KEYWORD_DENIED_EXECUTE_HANDLER,
+				    handler);
+	r->domain = domain;
+	return error;
 }
 
 /**
@@ -58,11 +64,9 @@ static int ccs_audit_execute_handler_log(struct ccs_execve_entry *ee,
  */
 static int ccs_audit_domain_creation_log(struct ccs_domain_info *domain)
 {
-	int error;
 	struct ccs_request_info r;
 	ccs_init_request_info(&r, domain, CCS_MAC_FILE_EXECUTE);
-	error = ccs_write_audit_log(false, &r, "use_profile %u\n", r.profile);
-	return error;
+	return ccs_write_audit_log(false, &r, "use_profile %u\n", r.profile);
 }
 
 /* The list for "struct ccs_domain_initializer_entry". */
