@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/internal.h
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.0   2009/09/04
+ * Version: 1.7.1+   2010/02/14
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -32,6 +32,11 @@
 #endif
 #include <linux/in6.h>
 #include <linux/ccsecurity.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
+#include <linux/err.h>
+#else
+#include <linux/fs.h>
+#endif
 #include "compat.h"
 
 /* Index numbers for Access Controls. */
@@ -45,9 +50,6 @@ enum ccs_acl_entry_type_index {
 	CCS_TYPE_IP_NETWORK_ACL,
 	CCS_TYPE_SIGNAL_ACL,
 	CCS_TYPE_MOUNT_ACL,
-	CCS_TYPE_UMOUNT_ACL,
-	CCS_TYPE_CHROOT_ACL,
-	CCS_TYPE_PIVOT_ROOT_ACL,
 	CCS_TYPE_EXECUTE_HANDLER,
 	CCS_TYPE_DENIED_EXECUTE_HANDLER
 };
@@ -71,6 +73,8 @@ enum ccs_path_acl_index {
 	CCS_TYPE_TRUNCATE,
 	CCS_TYPE_SYMLINK,
 	CCS_TYPE_REWRITE,
+	CCS_TYPE_CHROOT,
+	CCS_TYPE_UMOUNT,
 	CCS_MAX_PATH_OPERATION
 };
 
@@ -83,6 +87,7 @@ enum ccs_path_number3_acl_index {
 enum ccs_path2_acl_index {
 	CCS_TYPE_LINK,
 	CCS_TYPE_RENAME,
+	CCS_TYPE_PIVOT_ROOT,
 	CCS_MAX_PATH2_OPERATION
 };
 
@@ -256,7 +261,6 @@ enum ccs_conditions_index {
 #define CCS_KEYWORD_ADDRESS_GROUP             "address_group "
 #define CCS_KEYWORD_AGGREGATOR                "aggregator "
 #define CCS_KEYWORD_ALLOW_CAPABILITY          "allow_capability "
-#define CCS_KEYWORD_ALLOW_CHROOT              "allow_chroot "
 #define CCS_KEYWORD_ALLOW_ENV                 "allow_env "
 #define CCS_KEYWORD_ALLOW_IOCTL               "allow_ioctl "
 #define CCS_KEYWORD_ALLOW_CHMOD               "allow_chmod "
@@ -264,13 +268,11 @@ enum ccs_conditions_index {
 #define CCS_KEYWORD_ALLOW_CHGRP               "allow_chgrp "
 #define CCS_KEYWORD_ALLOW_MOUNT               "allow_mount "
 #define CCS_KEYWORD_ALLOW_NETWORK             "allow_network "
-#define CCS_KEYWORD_ALLOW_PIVOT_ROOT          "allow_pivot_root "
 #define CCS_KEYWORD_ALLOW_READ                "allow_read "
 #define CCS_KEYWORD_ALLOW_SIGNAL              "allow_signal "
 #define CCS_KEYWORD_DELETE                    "delete "
 #define CCS_KEYWORD_DENY_AUTOBIND             "deny_autobind "
 #define CCS_KEYWORD_DENY_REWRITE              "deny_rewrite "
-#define CCS_KEYWORD_ALLOW_UNMOUNT             "allow_unmount "
 #define CCS_KEYWORD_FILE_PATTERN              "file_pattern "
 #define CCS_KEYWORD_INITIALIZE_DOMAIN         "initialize_domain "
 #define CCS_KEYWORD_KEEP_DOMAIN               "keep_domain "
@@ -282,9 +284,10 @@ enum ccs_conditions_index {
 #define CCS_KEYWORD_USE_PROFILE               "use_profile "
 #define CCS_KEYWORD_IGNORE_GLOBAL_ALLOW_READ  "ignore_global_allow_read"
 #define CCS_KEYWORD_IGNORE_GLOBAL_ALLOW_ENV   "ignore_global_allow_env"
+#define CCS_KEYWORD_QUOTA_EXCEEDED            "quota_exceeded"
+#define CCS_KEYWORD_TRANSITION_FAILED         "transition_failed"
 #define CCS_KEYWORD_EXECUTE_HANDLER           "execute_handler"
 #define CCS_KEYWORD_DENIED_EXECUTE_HANDLER    "denied_execute_handler"
-#define CCS_KEYWORD_CAPABILITY                "capability::"
 
 /* A domain definition starts with <kernel>. */
 #define ROOT_NAME                         "<kernel>"
@@ -309,9 +312,9 @@ enum ccs_conditions_index {
 #define CCS_CONFIG_WANT_GRANT_LOG  128
 #define CCS_CONFIG_USE_DEFAULT     255
 
-/* Lowest 2 bits are reserved for open() mode. */
-#define CCS_USE_OPEN_MODE                 4
-#define CCS_CHECK_READ_FOR_OPEN_EXEC     16
+#define CCS_OPEN_FOR_READ_TRUNCATE        4
+#define CCS_OPEN_FOR_IOCTL_ONLY           8
+#define CCS_TASK_IS_IN_EXECVE            16
 #define CCS_DONT_SLEEP_ON_ENFORCE_ERROR  32
 #define CCS_TASK_IS_EXECUTE_HANDLER      64
 #define CCS_TASK_IS_POLICY_MANAGER      128
@@ -498,13 +501,10 @@ struct ccs_path_info {
 	u16 const_len;     /* = ccs_const_part_length(name)        */
 	bool is_dir;       /* = ccs_strendswith(name, "/")         */
 	bool is_patterned; /* = const_len < total_len              */
-	u16 depth;         /* = ccs_path_depth(name)               */
 };
 
 /* Structure for execve() operation. */
 struct ccs_execve_entry {
-	struct list_head list;
-	struct task_struct *task; /* = current */
 	struct ccs_request_info r;
 	struct ccs_obj_info obj;
 	struct linux_binprm *bprm;
@@ -605,19 +605,6 @@ struct ccs_aggregator_entry {
 	const struct ccs_path_info *aggregated_name;
 };
 
-/* Structure for "allow_unmount" keyword. */
-struct ccs_umount_acl {
-	struct ccs_acl_info head; /* type = CCS_TYPE_UMOUNT_ACL */
-	struct ccs_name_union dir;
-};
-
-/* Structure for "allow_pivot_root" keyword. */
-struct ccs_pivot_root_acl {
-	struct ccs_acl_info head; /* type = CCS_TYPE_PIVOT_ROOT_ACL */
-	struct ccs_name_union old_root;
-	struct ccs_name_union new_root;
-};
-
 /* Structure for "allow_mount" keyword. */
 struct ccs_mount_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_MOUNT_ACL */
@@ -627,12 +614,6 @@ struct ccs_mount_acl {
 	struct ccs_number_union flags;
 };
 
-/* Structure for "allow_chroot" keyword. */
-struct ccs_chroot_acl {
-	struct ccs_acl_info head; /* type = CCS_TYPE_CHROOT_ACL */
-	struct ccs_name_union dir;
-};
-
 /* Structure for "deny_autobind" keyword. */
 struct ccs_reserved_entry {
 	struct list_head list;
@@ -677,7 +658,7 @@ struct ccs_envp_entry {
  * and executes the original execve() requests if appropriate.
  *
  * "denied_execute_handler" directive is used only when execve() request was
- * rejected in enforcing mode (i.e. MAC_FOR_FILE=enforcing).
+ * rejected in enforcing mode (i.e. CONFIG::file::execute={ mode=enforcing }).
  * The program specified by "denied_execute_handler" does whatever it wants
  * to do (e.g. silently terminate, change firewall settings,
  * redirect the user to honey pot etc.).
@@ -690,7 +671,8 @@ struct ccs_execute_handler_record {
 /*
  * Structure for "allow_read/write", "allow_execute", "allow_read",
  * "allow_write", "allow_unlink", "allow_rmdir", "allow_truncate",
- * "allow_symlink" and "allow_rewrite" directive.
+ * "allow_symlink", "allow_rewrite", "allow_chroot" and "allow_unmount"
+ * directive.
  */
 struct ccs_path_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_PATH_ACL */
@@ -708,7 +690,9 @@ struct ccs_path_number3_acl {
 	struct ccs_number_union minor;
 };
 
-/* Structure for "allow_rename" and "allow_link" directive. */
+/*
+ * Structure for "allow_rename", "allow_link" and "allow_pivot_root" directive.
+ */
 struct ccs_path2_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_PATH2_ACL */
 	u8 perm;
@@ -786,6 +770,13 @@ struct ccs_ip_network_acl {
 	struct ccs_number_union port;
 };
 
+/* Structure for string data. */
+struct ccs_name_entry {
+	struct list_head list;
+	atomic_t users;
+	int size;
+	struct ccs_path_info entry;
+};
 
 /* Structure for reading/writing policy via /proc interfaces. */
 struct ccs_io_buffer {
@@ -835,6 +826,8 @@ struct ccs_preference {
 #endif
 	unsigned int learning_max_entry;
 	unsigned int enforcing_penalty;
+	bool audit_task_info;
+	bool audit_path_info;
 	bool enforcing_verbose;
 	bool learning_verbose;
 	bool learning_exec_realpath;
@@ -845,9 +838,7 @@ struct ccs_preference {
 
 struct ccs_profile {
 	const struct ccs_path_info *comment;
-#ifdef CONFIG_CCSECURITY_AUDIT
 	struct ccs_preference *audit;
-#endif
 	struct ccs_preference *learning;
 	struct ccs_preference *permissive;
 	struct ccs_preference *enforcing;
@@ -928,8 +919,7 @@ int ccs_init_request_info(struct ccs_request_info *r,
 int ccs_open_control(const u8 type, struct file *file);
 int ccs_parse_ip_address(char *address, u16 *min, u16 *max);
 int ccs_poll_control(struct file *file, poll_table *wait);
-int ccs_poll_grant_log(struct file *file, poll_table *wait);
-int ccs_poll_reject_log(struct file *file, poll_table *wait);
+int ccs_poll_audit_log(struct file *file, poll_table *wait);
 int ccs_read_control(struct file *file, char __user *buffer,
 		     const int buffer_len);
 int ccs_read_lock(void);
@@ -944,9 +934,6 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 int ccs_write_capability_policy(char *data, struct ccs_domain_info *domain,
 				struct ccs_condition *condition,
 				const bool is_delete);
-int ccs_write_chroot_policy(char *data, struct ccs_domain_info *domain,
-			    struct ccs_condition *condition,
-			    const bool is_delete);
 int ccs_write_control(struct file *file, const char __user *buffer,
 		      const int buffer_len);
 int ccs_write_domain_initializer_policy(char *data, const bool is_not,
@@ -972,16 +959,11 @@ int ccs_write_no_rewrite_policy(char *data, const bool is_delete);
 int ccs_write_number_group_policy(char *data, const bool is_delete);
 int ccs_write_path_group_policy(char *data, const bool is_delete);
 int ccs_write_pattern_policy(char *data, const bool is_delete);
-int ccs_write_pivot_root_policy(char *data, struct ccs_domain_info *domain,
-				struct ccs_condition *condition,
-				const bool is_delete);
 int ccs_write_reserved_port_policy(char *data, const bool is_delete);
 int ccs_write_signal_policy(char *data, struct ccs_domain_info *domain,
 			    struct ccs_condition *condition,
 			    const bool is_delete);
-int ccs_write_umount_policy(char *data, struct ccs_domain_info *domain,
-			    struct ccs_condition *condition,
-			    const bool is_delete);
+size_t ccs_del_condition(struct ccs_condition *cond);
 struct ccs_address_group *ccs_get_address_group(const char *group_name);
 struct ccs_condition *ccs_get_condition(char * const condition);
 struct ccs_domain_info *ccs_find_domain(const char *domainname);
@@ -993,6 +975,7 @@ struct ccs_profile *ccs_profile(const u8 profile);
 u8 ccs_parse_ulong(unsigned long *result, char **str);
 void ccs_check_profile(void);
 void ccs_fill_path_info(struct ccs_path_info *ptr);
+void ccs_get_attributes(struct ccs_obj_info *obj);
 void ccs_load_policy(const char *filename);
 void ccs_memory_free(const void *ptr, size_t size);
 void ccs_normalize_line(unsigned char *buffer);
@@ -1000,17 +983,10 @@ void ccs_print_ipv6(char *buffer, const int buffer_len,
 		    const struct in6_addr *ip);
 void ccs_print_ulong(char *buffer, const int buffer_len,
 		     const unsigned long value, const u8 type);
-void ccs_put_address_group(struct ccs_address_group *group);
-void ccs_put_condition(struct ccs_condition *cond);
-void ccs_put_ipv6_address(const struct in6_addr *addr);
-void ccs_put_name(const struct ccs_path_info *name);
 void ccs_put_name_union(struct ccs_name_union *ptr);
-void ccs_put_number_group(struct ccs_number_group *group);
 void ccs_put_number_union(struct ccs_number_union *ptr);
-void ccs_put_path_group(struct ccs_path_group *group);
-void ccs_read_grant_log(struct ccs_io_buffer *head);
+void ccs_read_audit_log(struct ccs_io_buffer *head);
 void ccs_read_memory_counter(struct ccs_io_buffer *head);
-void ccs_read_reject_log(struct ccs_io_buffer *head);
 void ccs_read_unlock(const int idx);
 void ccs_run_gc(void);
 void ccs_warn_log(struct ccs_request_info *r, const char *fmt, ...)
@@ -1024,12 +1000,16 @@ static inline bool ccs_pathcmp(const struct ccs_path_info *a,
 	return a->hash != b->hash || strcmp(a->name, b->name);
 }
 
-static inline int ccs_memcmp(void *a, void *b, const u8 offset, const u8 size)
+static inline int ccs_memcmp(void *a, void *b, const size_t offset,
+			     const size_t size)
 {
 	return memcmp(((char *) a) + offset, ((char *) b) + offset,
 		      size - offset);
 }
 
+#define CCS_HASH_BITS 8
+#define CCS_MAX_HASH (1 << CCS_HASH_BITS)
+
 extern struct mutex ccs_policy_lock;
 extern struct list_head ccs_domain_list;
 extern struct list_head ccs_address_group_list;
@@ -1044,9 +1024,10 @@ extern struct list_head ccs_domain_keeper_list;
 extern struct list_head ccs_aggregator_list;
 extern struct list_head ccs_reservedport_list;
 extern struct list_head ccs_policy_manager_list;
-
-extern spinlock_t ccs_execve_list_lock;
-extern unsigned int ccs_in_execve_counter;
+extern struct list_head ccs_address_list;
+extern struct list_head ccs_condition_list;
+extern struct mutex ccs_name_list_lock;
+extern struct list_head ccs_name_list[CCS_MAX_HASH];
 
 extern bool ccs_policy_loaded;
 extern struct ccs_domain_info ccs_kernel_domain;
@@ -1100,10 +1081,34 @@ static inline void ccs_realpath_unlock(void)
 
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
+
+static inline void ccs_tasklist_lock(void)
+{
+	rcu_read_lock();
+}
+static inline void ccs_tasklist_unlock(void)
+{
+	rcu_read_unlock();
+}
+
+#else
+
+static inline void ccs_tasklist_lock(void)
+{
+	read_lock(&tasklist_lock);
+}
+static inline void ccs_tasklist_unlock(void)
+{
+	read_unlock(&tasklist_lock);
+}
+
+#endif
+
 static inline struct ccs_domain_info *ccs_task_domain(struct task_struct *task)
 {
-	return task->ccs_domain_info ?
-		task->ccs_domain_info : &ccs_kernel_domain;
+	struct ccs_domain_info *domain = task->ccs_domain_info;
+	return domain ? domain : &ccs_kernel_domain;
 }
 
 static inline struct ccs_domain_info *ccs_current_domain(void)
@@ -1143,4 +1148,46 @@ static inline int ccs_round2(size_t size)
 }
 #endif
 
+static inline void ccs_put_path_group(struct ccs_path_group *group)
+{
+	if (group)
+		atomic_dec(&group->users);
+}
+
+static inline void ccs_put_number_group(struct ccs_number_group *group)
+{
+	if (group)
+		atomic_dec(&group->users);
+}
+
+static inline void ccs_put_address_group(struct ccs_address_group *group)
+{
+	if (group)
+		atomic_dec(&group->users);
+}
+
+static inline void ccs_put_ipv6_address(const struct in6_addr *addr)
+{
+	if (addr) {
+		struct ccs_ipv6addr_entry *ptr =
+			container_of(addr, struct ccs_ipv6addr_entry, addr);
+		atomic_dec(&ptr->users);
+	}
+}
+
+static inline void ccs_put_condition(struct ccs_condition *cond)
+{
+	if (cond)
+		atomic_dec(&cond->users);
+}
+
+static inline void ccs_put_name(const struct ccs_path_info *name)
+{
+	if (name) {
+		struct ccs_name_entry *ptr =
+			container_of(name, struct ccs_name_entry, entry);
+		atomic_dec(&ptr->users);
+	}
+}
+
 #endif
