@@ -1,9 +1,9 @@
 /*
- * security/ccsecurity/gc.c
+ * security/ccsecurity/memory.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.0   2009/09/03
+ * Version: 1.7.1+   2010/03/26
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -11,6 +11,9 @@
  */
 
 #include "internal.h"
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) || defined(RHEL_MAJOR)
+#include <linux/hash.h>
+#endif
 
 void ccs_warn_oom(const char *function)
 {
@@ -35,7 +38,7 @@ static unsigned int ccs_quota_for_policy;
  * @ptr:  Pointer to allocated memory.
  * @size: Size in byte.
  *
- * Returns true if @ptr is not NULL and quota not exceeded, false otehrwise.
+ * Returns true if @ptr is not NULL and quota not exceeded, false otherwise.
  */
 bool ccs_memory_ok(const void *ptr, const unsigned int size)
 {
@@ -57,7 +60,7 @@ bool ccs_memory_ok(const void *ptr, const unsigned int size)
  * @data:   Data to copy from.
  * @size:   Size in byte.
  *
- * Returns true if @ptr is not NULL and quota not exceeded, false otehrwise.
+ * Returns true if @ptr is not NULL and quota not exceeded, false otherwise.
  */
 bool ccs_commit_ok(void *ptr, void *data, const unsigned int size)
 {
@@ -82,7 +85,7 @@ void ccs_memory_free(const void *ptr, size_t size)
 	kfree(ptr);
 }
 
-static LIST_HEAD(ccs_address_list);
+LIST_HEAD(ccs_address_list);
 
 /**
  * ccs_get_ipv6_address - Keep the given IPv6 address on the RAM.
@@ -115,100 +118,16 @@ const struct in6_addr *ccs_get_ipv6_address(const struct in6_addr *addr)
 		atomic_set(&ptr->users, 1);
 		list_add_tail(&ptr->list, &ccs_address_list);
 		entry = NULL;
+		error = 0;
 	}
 	mutex_unlock(&ccs_policy_lock);
 	kfree(entry);
-	return ptr ? &ptr->addr : NULL;
+	return !error ? &ptr->addr : NULL;
 }
 
-/**
- * ccs_put_ipv6_address - Delete the given IPv6 address on the RAM.
- *
- * @addr: Pointer to "struct in6_addr".
- */
-void ccs_put_ipv6_address(const struct in6_addr *addr)
-{
-	struct ccs_ipv6addr_entry *ptr;
-	bool can_delete = false;
-	if (!addr)
-		return;
-	ptr = container_of(addr, struct ccs_ipv6addr_entry, addr);
-	mutex_lock(&ccs_policy_lock);
-	if (atomic_dec_and_test(&ptr->users)) {
-		list_del(&ptr->list);
-		can_delete = true;
-	}
-	mutex_unlock(&ccs_policy_lock);
-	if (can_delete)
-		ccs_memory_free(ptr, sizeof(*ptr));
-}
-
-/**
- * ccs_put_condition - Delete memory for "struct ccs_condition".
- *
- * @cond: Pointer to "struct ccs_condition".
- */
-void ccs_put_condition(struct ccs_condition *cond)
-{
-	const struct ccs_condition_element *condp;
-	struct ccs_number_union *numbers_p;
-	struct ccs_name_union *names_p;
-	const struct ccs_argv_entry *argv;
-	const struct ccs_envp_entry *envp;
-	u16 condc;
-	u16 numbers_count;
-	u16 names_count;
-	u16 argc;
-	u16 envc;
-	u16 i;
-	bool can_delete = false;
-	if (!cond)
-		return;
-	BUG_ON(atomic_read(&cond->users) <= 0);
-	mutex_lock(&ccs_policy_lock);
-	if (atomic_dec_and_test(&cond->users)) {
-		list_del(&cond->list);
-		can_delete = true;
-	}
-	mutex_unlock(&ccs_policy_lock);
-	if (!can_delete)
-		return;
-	condc = cond->condc;
-	numbers_count = cond->numbers_count;
-	names_count = cond->names_count;
-	argc = cond->argc;
-	envc = cond->envc;
-	condp = (const struct ccs_condition_element *) (cond + 1);
-	numbers_p = (struct ccs_number_union *) (condp + condc);
-	names_p = (struct ccs_name_union *) (numbers_p + numbers_count);
-	argv = (const struct ccs_argv_entry *) (names_p + names_count);
-	envp = (const struct ccs_envp_entry *) (argv + argc);
-	for (i = 0; i < numbers_count; i++)
-		ccs_put_number_union(numbers_p++);
-	for (i = 0; i < names_count; i++)
-		ccs_put_name_union(names_p++);
-	for (i = 0; i < argc; argv++, i++)
-		ccs_put_name(argv->value);
-	for (i = 0; i < envc; envp++, i++) {
-		ccs_put_name(envp->name);
-		ccs_put_name(envp->value);
-	}
-	ccs_memory_free(cond, cond->size);
-}
-
-#define CCS_MAX_HASH 256
-
-/* Structure for string data. */
-struct ccs_name_entry {
-	struct list_head list;
-	atomic_t users;
-	int size;
-	struct ccs_path_info entry;
-};
-
 /* The list for "struct ccs_name_entry". */
-static struct list_head ccs_name_list[CCS_MAX_HASH];
-static DEFINE_MUTEX(ccs_name_list_lock);
+struct list_head ccs_name_list[CCS_MAX_HASH];
+DEFINE_MUTEX(ccs_name_list_lock);
 
 /**
  * ccs_get_name - Allocate memory for string data.
@@ -223,13 +142,19 @@ const struct ccs_path_info *ccs_get_name(const char *name)
 	unsigned int hash;
 	int len;
 	int allocated_len;
+	struct list_head *head;
 
 	if (!name)
 		return NULL;
 	len = strlen(name) + 1;
 	hash = full_name_hash((const unsigned char *) name, len - 1);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) || defined(RHEL_MAJOR)
+	head = &ccs_name_list[hash_long(hash, CCS_HASH_BITS)];
+#else
+	head = &ccs_name_list[hash % CCS_MAX_HASH];
+#endif
 	mutex_lock(&ccs_name_list_lock);
-	list_for_each_entry(ptr, &ccs_name_list[hash % CCS_MAX_HASH], list) {
+	list_for_each_entry(ptr, head, list) {
 		if (hash != ptr->entry.hash || strcmp(name, ptr->entry.name))
 			continue;
 		atomic_inc(&ptr->users);
@@ -251,42 +176,18 @@ const struct ccs_path_info *ccs_get_name(const char *name)
 	atomic_set(&ptr->users, 1);
 	ccs_fill_path_info(&ptr->entry);
 	ptr->size = allocated_len;
-	list_add_tail(&ptr->list, &ccs_name_list[hash % CCS_MAX_HASH]);
+	list_add_tail(&ptr->list, head);
  out:
 	mutex_unlock(&ccs_name_list_lock);
 	return ptr ? &ptr->entry : NULL;
 }
 
 /**
- * ccs_put_name - Delete shared memory for string data.
- *
- * @name: Pointer to "struct ccs_path_info".
- */
-void ccs_put_name(const struct ccs_path_info *name)
-{
-	struct ccs_name_entry *ptr;
-	bool can_delete = false;
-	if (!name)
-		return;
-	ptr = container_of(name, struct ccs_name_entry, entry);
-	mutex_lock(&ccs_name_list_lock);
-	if (atomic_dec_and_test(&ptr->users)) {
-		list_del(&ptr->list);
-		can_delete = true;
-	}
-	mutex_unlock(&ccs_name_list_lock);
-	if (can_delete) {
-		atomic_sub(ptr->size, &ccs_policy_memory_size);
-		kfree(ptr);
-	}
-}
-
-/**
- * ccs_realpath_init - Initialize realpath related code.
+ * ccs_mm_init - Initialize mm related code.
  *
  * Returns 0.
  */
-static int __init ccs_realpath_init(void)
+static int __init ccs_mm_init(void)
 {
 	int i;
 	for (i = 0; i < CCS_MAX_HASH; i++)
@@ -296,7 +197,6 @@ static int __init ccs_realpath_init(void)
 	list_add_tail_rcu(&ccs_kernel_domain.list, &ccs_domain_list);
 	if (ccs_find_domain(ROOT_NAME) != &ccs_kernel_domain)
 		panic("Can't register ccs_kernel_domain");
-#ifdef CONFIG_CCSECURITY_BUILTIN_INITIALIZERS
 	{
 		/* Load built-in policy. */
 		static char ccs_builtin_initializers[] __initdata
@@ -311,14 +211,13 @@ static int __init ccs_realpath_init(void)
 			cp = cp2;
 		}
 	}
-#endif
 	return 0;
 }
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-__initcall(ccs_realpath_init);
+__initcall(ccs_mm_init);
 #else
-core_initcall(ccs_realpath_init);
+core_initcall(ccs_mm_init);
 #endif
 
 unsigned int ccs_audit_log_memory_size;
