@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2009  NTT DATA CORPORATION
  *
- * Version: 1.7.0   2009/09/03
+ * Version: 1.7.1+   2009/12/20
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -199,6 +199,27 @@ char *ccs_realpath_from_path(struct path *path)
 		path_put(path);
 		if (IS_ERR(pos))
 			continue;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 34)
+		/* Prepend "/proc" prefix if using internal proc vfs mount. */
+		if (path->mnt->mnt_flags & MNT_INTERNAL &&
+		    path->mnt->mnt_sb->s_magic == PROC_SUPER_MAGIC) {
+			pos -= 5;
+			if (pos >= buf)
+				memmove(pos, "/proc", 5);
+			else
+				continue;
+		}
+#elif LINUX_VERSION_CODE == KERNEL_VERSION(2, 6, 33)
+		/* Prepend "/proc" prefix if using internal proc vfs mount. */
+		if (path->mnt->mnt_parent == path->mnt &&
+		    path->mnt->mnt_sb->s_magic == PROC_SUPER_MAGIC) {
+			pos -= 5;
+			if (pos >= buf)
+				memmove(pos, "/proc", 5);
+			else
+				continue;
+		}
+#endif
 		name = ccs_encode(pos);
 		break;
 	}
