@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2009  NTT DATA CORPORATION
  *
- * Version: 1.7.0   2009/09/03
+ * Version: 1.7.1+   2009/12/20
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -33,43 +33,43 @@ struct socket;
 struct sockaddr;
 struct sock;
 struct sk_buff;
+struct msghdr;
 
 #if defined(CONFIG_CCSECURITY)
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
 /* Check whether the given pathname is allowed to chroot to. */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
 int ccs_chroot_permission(struct path *path);
-#else
-int ccs_chroot_permission(struct nameidata *nd);
-#endif
-
-/* Check whether the mount operation with the given parameters is allowed. */
-int ccs_mount_permission(char *dev_name, char *dir_name, char *type,
-			 const unsigned long *flags);
-
 /* Check whether the current process is allowed to pivot_root. */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
 int ccs_pivot_root_permission(struct path *old_path, struct path *new_path);
-#else
-int ccs_pivot_root_permission(struct nameidata *old_nd,
-			      struct nameidata *new_nd);
-#endif
-
 /* Check whether the given mount operation hides an mounted partition. */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
 int ccs_may_mount(struct path *path);
 #else
+int ccs_chroot_permission(struct nameidata *nd);
+int ccs_pivot_root_permission(struct nameidata *old_nd,
+			      struct nameidata *new_nd);
 int ccs_may_mount(struct nameidata *nd);
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+/* Check whether the mount operation with the given parameters is allowed. */
+int ccs_mount_permission(char *dev_name, struct path *dir, char *type,
+			 unsigned long flags, void *data_page);
+#else
+int ccs_mount_permission(char *dev_name, struct nameidata *nd, char *type,
+			 unsigned long flags, void *data_page);
+#endif
+
 /* Check whether the given mountpoint is allowed to umount. */
-int ccs_may_umount(struct vfsmount *mnt);
+int ccs_umount_permission(struct vfsmount *mnt, int flags);
 
 /* Check whether the given local port is reserved. */
 _Bool ccs_lport_reserved(const u16 port);
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 34)
 void ccs_save_open_mode(int mode);
 void ccs_clear_open_mode(void);
+#endif
 int ccs_open_permission(struct dentry *dentry, struct vfsmount *mnt,
 			const int flag);
 int ccs_rewrite_permission(struct file *filp);
@@ -91,7 +91,7 @@ int ccs_rmdir_permission(struct inode *dir, struct dentry *dentry,
 int ccs_unlink_permission(struct inode *dir, struct dentry *dentry,
 			  struct vfsmount *mnt);
 int ccs_symlink_permission(struct inode *dir, struct dentry *dentry,
-			   struct vfsmount *mnt, char *from);
+			   struct vfsmount *mnt, const char *from);
 int ccs_truncate_permission(struct dentry *dentry, struct vfsmount *mnt,
 			    loff_t length, unsigned int time_attrs);
 int ccs_rename_permission(struct inode *old_dir, struct dentry *old_dentry,
@@ -101,7 +101,6 @@ int ccs_link_permission(struct dentry *old_dentry, struct inode *new_dir,
 			struct dentry *new_dentry, struct vfsmount *mnt);
 int ccs_open_exec_permission(struct dentry *dentry, struct vfsmount *mnt);
 int ccs_uselib_permission(struct dentry *dentry, struct vfsmount *mnt);
-int ccs_setattr_permission(struct dentry *dentry, struct iattr *attr);
 int ccs_kill_permission(pid_t pid, int sig);
 int ccs_tgkill_permission(pid_t tgid, pid_t pid, int sig);
 int ccs_tkill_permission(pid_t pid, int sig);
@@ -113,8 +112,8 @@ int ccs_socket_connect_permission(struct socket *sock, struct sockaddr *addr,
 int ccs_socket_bind_permission(struct socket *sock, struct sockaddr *addr,
 			       int addr_len);
 int ccs_socket_accept_permission(struct socket *sock, struct sockaddr *addr);
-int ccs_socket_sendmsg_permission(struct socket *sock, struct sockaddr *addr,
-				  int addr_len);
+int ccs_socket_sendmsg_permission(struct socket *sock, struct msghdr *msg,
+				  int size);
 int ccs_socket_recvmsg_permission(struct sock *sk, struct sk_buff *skb,
 				  const unsigned int flags);
 int ccs_chown_permission(struct dentry *dentry, struct vfsmount *mnt,
@@ -131,50 +130,62 @@ static inline int ccs_chroot_permission(struct path *path)
 {
 	return 0;
 }
-#else
-static inline int ccs_chroot_permission(struct nameidata *nd)
-{
-	return 0;
-}
-#endif
-static inline int ccs_mount_permission(char *dev_name, char *dir_name,
-				       char *type, const unsigned long *flags)
-{
-	return 0;
-}
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+
 static inline int ccs_pivot_root_permission(struct path *old_path,
 					    struct path *new_path)
 {
 	return 0;
 }
-#else
-static inline int ccs_pivot_root_permission(struct nameidata *old_nd,
-					    struct nameidata *new_nd)
-{
-	return 0;
-}
-#endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+
 static inline int ccs_may_mount(struct path *path)
 {
 	return 0;
 }
 #else
+static inline int ccs_chroot_permission(struct nameidata *nd)
+{
+	return 0;
+}
+
+static inline int ccs_pivot_root_permission(struct nameidata *old_nd,
+					    struct nameidata *new_nd)
+{
+	return 0;
+}
+
 static inline int ccs_may_mount(struct nameidata *nd)
 {
 	return 0;
 }
 #endif
-static inline int ccs_may_umount(struct vfsmount *mnt)
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+static inline int ccs_mount_permission(char *dev_name, struct path *dir,
+				       char *type, unsigned long flags,
+				       void *data_page)
 {
 	return 0;
 }
+#else
+static inline int ccs_mount_permission(char *dev_name, struct nameidata *nd,
+				       char *type, unsigned long flags,
+				       void *data_page)
+{
+	return 0;
+}
+#endif
+
+static inline int ccs_umount_permission(struct vfsmount *mnt, int flags)
+{
+	return 0;
+}
+
 static inline _Bool ccs_lport_reserved(const u16 port)
 {
 	return 0;
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 34)
 static inline void ccs_save_open_mode(int mode)
 {
 }
@@ -182,6 +193,7 @@ static inline void ccs_save_open_mode(int mode)
 static inline void ccs_clear_open_mode(void)
 {
 }
+#endif
 
 static inline int ccs_open_permission(struct dentry *dentry,
 				      struct vfsmount *mnt, const int flag)
@@ -243,7 +255,8 @@ static inline int ccs_unlink_permission(struct inode *dir,
 
 static inline int ccs_symlink_permission(struct inode *dir,
 					 struct dentry *dentry,
-					 struct vfsmount *mnt, char *from)
+					 struct vfsmount *mnt,
+					 const char *from)
 {
 	return 0;
 }
@@ -284,12 +297,6 @@ static inline int ccs_uselib_permission(struct dentry *dentry,
 	return 0;
 }
 
-static inline int ccs_setattr_permission(struct dentry *dentry,
-					 struct iattr *attr)
-{
-	return 0;
-}
-
 static inline int ccs_kill_permission(pid_t pid, int sig)
 {
 	return 0;
@@ -337,8 +344,7 @@ static inline int ccs_socket_accept_permission(struct socket *sock,
 }
 
 static inline int ccs_socket_sendmsg_permission(struct socket *sock,
-						struct sockaddr *addr,
-						int addr_len)
+						struct msghdr *msg, int size)
 {
 	return 0;
 }
@@ -379,27 +385,13 @@ static inline int ccs_tgsigqueue_permission(pid_t tgid, pid_t pid, int sig)
 int ccs_may_create(struct inode *dir, struct dentry *dentry);
 int ccs_may_delete(struct inode *dir, struct dentry *dentry, int is_dir);
 #else
-/* SUSE 11.0 adds is_dir for may_create(). */
-#ifdef MS_WITHAPPEND
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 27)
-int ccs_may_create(struct inode *dir, struct dentry *dentry,
-		   struct nameidata *nd, int is_dir);
-#else
 int ccs_may_create(struct inode *dir, struct dentry *dentry, int is_dir);
-#endif
-#else
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 27)
-int ccs_may_create(struct inode *dir, struct dentry *dentry,
-		   struct nameidata *nd);
-#else
-int ccs_may_create(struct inode *dir, struct dentry *dentry);
-#endif
-#endif
 int ccs_may_delete(struct inode *dir, struct dentry *dentry, int is_dir);
 #endif
 
-int ccs_start_execve(struct linux_binprm *bprm);
-void ccs_finish_execve(int retval);
+struct ccs_execve_entry;
+int ccs_start_execve(struct linux_binprm *bprm, struct ccs_execve_entry **ee);
+void ccs_finish_execve(int retval, struct ccs_execve_entry *ee);
 
 int search_binary_handler(struct linux_binprm *, struct pt_regs *);
 
@@ -407,11 +399,11 @@ int search_binary_handler(struct linux_binprm *, struct pt_regs *);
 static inline int ccs_search_binary_handler(struct linux_binprm *bprm,
 					    struct pt_regs *regs)
 {
-	int retval = ccs_start_execve(bprm);
-	if (!retval) {
+	struct ccs_execve_entry *ee;
+	int retval = ccs_start_execve(bprm, &ee);
+	if (!retval)
 		retval = search_binary_handler(bprm, regs);
-		ccs_finish_execve(retval);
-	}
+	ccs_finish_execve(retval, ee);
 	return retval;
 }
 #else
@@ -487,4 +479,24 @@ enum ccs_capability_acl_index {
 	CCS_MAX_CAPABILITY_INDEX
 };
 
+static inline int ccs_ptrace_permission(long request, long pid)
+{
+	return !ccs_capable(CCS_SYS_PTRACE);
+}
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
+#ifdef F_SETFL
+
+static inline int ccs_fcntl_permission(struct file *file, unsigned int cmd,
+				       unsigned long arg)
+{
+	if (cmd == F_SETFL && ((arg ^ file->f_flags) & O_APPEND) &&
+	    ccs_rewrite_permission(file))
+		return -EPERM;
+	return 0;
+}
+
+#endif
+#endif
+
 #endif
